@@ -1753,8 +1753,10 @@ nmc_do_cmd (NmCli *nmc, const NMCCommand cmds[], const char *cmd, int argc, char
 				c->usage ();
 			g_simple_async_result_complete_in_idle (simple);
 			g_object_unref (simple);
-		} else
-			call_cmd (nmc, simple, c, argc-1, argv+1);
+		} else {
+			next_arg (nmc, &argc, &argv);
+			call_cmd (nmc, simple, c, argc, argv);
+		}
 	} else if (cmd) {
 		/* Not a known command. */
 		if (nmc_arg_is_help (cmd) && c->usage) {
