@@ -385,13 +385,6 @@ show_nm_status (NmCli *nmc, const char *pretty_header_name, const char *print_fl
 static NMCResultCode
 do_general_status (NmCli *nmc, int argc, char **argv)
 {
-	gs_free_error GError *error = NULL;
-
-        if (!nmc_terse_option_check (nmc->print_output, nmc->required_fields, &error)) {
-                g_string_printf (nmc->return_text, _("Error: %s."), error->message);
-                return NMC_RESULT_ERROR_USER_INPUT;
-	}
-
 	if (nmc->complete)
 		return nmc->return_value;
 
@@ -566,13 +559,6 @@ show_nm_permissions (NmCli *nmc)
 static NMCResultCode
 do_general_permissions (NmCli *nmc, int argc, char **argv)
 {
-	gs_free_error GError *error = NULL;
-
-        if (!nmc_terse_option_check (nmc->print_output, nmc->required_fields, &error)) {
-                g_string_printf (nmc->return_text, _("Error: %s."), error->message);
-                return NMC_RESULT_ERROR_USER_INPUT;
-	}
-
 	if (nmc->complete)
 		return nmc->return_value;
 
@@ -655,12 +641,6 @@ do_general_logging (NmCli *nmc, int argc, char **argv)
 	gs_free_error GError *error = NULL;
 
 	if (argc == 0) {
-		if (!nmc_terse_option_check (nmc->print_output, nmc->required_fields, &error)) {
-			g_string_printf (nmc->return_text, _("Error: %s."), error->message);
-			g_error_free (error);
-			return NMC_RESULT_ERROR_USER_INPUT;
-		}
-
 		if (nmc->complete)
 			return nmc->return_value;
 
@@ -675,20 +655,24 @@ do_general_logging (NmCli *nmc, int argc, char **argv)
 				nmc_complete_strings (*argv, "level", "domains", NULL);
 
 			if (matches (*argv, "level")) {
-				if (next_arg (&argc, &argv) != 0) {
+				if (!argc) {
 					g_string_printf (nmc->return_text, _("Error: '%s' argument is missing."), *(argv-1));
 					return NMC_RESULT_ERROR_USER_INPUT;
 				}
+				argc--;
+				argv++;
 				if (argc == 1 && nmc->complete) {
 					nmc_complete_strings_nocase (*argv, "TRACE", "DEBUG", "INFO", "WARN",
 					                             "ERR", "OFF", "KEEP", NULL);
 				}
 				level = *argv;
 			} else if (matches (*argv, "domains")) {
-				if (next_arg (&argc, &argv) != 0) {
+				if (!argc) {
 					g_string_printf (nmc->return_text, _("Error: '%s' argument is missing."), *(argv-1));
 					return NMC_RESULT_ERROR_USER_INPUT;
 				}
+				argc--;
+				argv++;
 				if (argc == 1 && nmc->complete) {
 					nmc_complete_strings_nocase (*argv, "PLATFORM", "RFKILL", "ETHER", "WIFI", "BT",
 					                             "MB", "DHCP4", "DHCP6", "PPP", "WIFI_SCAN", "IP4",
@@ -703,7 +687,7 @@ do_general_logging (NmCli *nmc, int argc, char **argv)
 				g_string_printf (nmc->return_text, _("Error: property '%s' is not known."), *argv);
 				return NMC_RESULT_ERROR_USER_INPUT;
 			}
-		} while (next_arg (&argc, &argv) == 0);
+		} while (next_arg (nmc, &argc, &argv) == 0);
 
 		if (nmc->complete)
 			return nmc->return_value;
@@ -753,7 +737,7 @@ do_general_hostname (NmCli *nmc, int argc, char **argv)
 		/* hostname provided -> set it */
 		const char *hostname = *argv;
 
-		if (next_arg (&argc, &argv) == 0)
+		if (next_arg (nmc, &argc, &argv) == 0)
 			g_print ("Warning: ignoring extra garbage after '%s' hostname\n", hostname);
 
 		nmc->should_wait++;
@@ -918,17 +902,12 @@ static NMCResultCode
 do_radio_all (NmCli *nmc, int argc, char **argv)
 {
 	gboolean enable_flag;
-	gs_free_error GError *error = NULL;
 
 	if (argc == 0) {
 		if (nmc->complete)
 			return nmc->return_value;
 
 		/* no argument, show all radio switches */
-		if (!nmc_terse_option_check (nmc->print_output, nmc->required_fields, &error)) {
-			g_string_printf (nmc->return_text, _("Error: %s."), error->message);
-			return NMC_RESULT_ERROR_USER_INPUT;
-		}
 		show_nm_status (nmc, _("Radio switches"), NMC_FIELDS_NM_STATUS_RADIO);
 	} else {
 		if (nmc->complete) {
