@@ -131,12 +131,12 @@ complete_fields (const char *prefix)
 	complete_field (h, NULL, nmc_fields_dev_lldp_list);
 
 	complete_field (h, "connection", nmc_fields_setting_connection);
-	complete_field (h, "wired", nmc_fields_setting_wired);
-	complete_field (h, "8021X", nmc_fields_setting_8021X);
-	complete_field (h, "wireless", nmc_fields_setting_wireless);
-	complete_field (h, "wireless_security", nmc_fields_setting_wireless_security);
-	complete_field (h, "ip4-config", nmc_fields_setting_ip4_config);
-	complete_field (h, "ip6-config", nmc_fields_setting_ip6_config);
+	complete_field (h, "802-3-ethernet", nmc_fields_setting_wired);
+	complete_field (h, "802-1x", nmc_fields_setting_8021X);
+	complete_field (h, "802-11-wireless", nmc_fields_setting_wireless);
+	complete_field (h, "802-11-wireless-security", nmc_fields_setting_wireless_security);
+	complete_field (h, "ipv4", nmc_fields_setting_ip4_config);
+	complete_field (h, "ipv6", nmc_fields_setting_ip6_config);
 	complete_field (h, "serial", nmc_fields_setting_serial);
 	complete_field (h, "ppp", nmc_fields_setting_ppp);
 	complete_field (h, "pppoe", nmc_fields_setting_pppoe);
@@ -144,7 +144,7 @@ complete_fields (const char *prefix)
 	complete_field (h, "gsm", nmc_fields_setting_gsm);
 	complete_field (h, "cdma", nmc_fields_setting_cdma);
 	complete_field (h, "bluetooth", nmc_fields_setting_bluetooth);
-	complete_field (h, "olpc-mesh", nmc_fields_setting_olpc_mesh);
+	complete_field (h, "802-11-olpc-mesh", nmc_fields_setting_olpc_mesh);
 	complete_field (h, "vpn", nmc_fields_setting_vpn);
 	complete_field (h, "wimax", nmc_fields_setting_wimax);
 	complete_field (h, "infiniband", nmc_fields_setting_infiniband);
@@ -153,7 +153,7 @@ complete_fields (const char *prefix)
 	complete_field (h, "bridge", nmc_fields_setting_bridge);
 	complete_field (h, "bridge-port", nmc_fields_setting_bridge_port);
 	complete_field (h, "team", nmc_fields_setting_team);
-	complete_field (h, "team0port", nmc_fields_setting_team_port);
+	complete_field (h, "team-port", nmc_fields_setting_team_port);
 	complete_field (h, "dcb", nmc_fields_setting_dcb);
 	complete_field (h, "tun", nmc_fields_setting_tun);
 	complete_field (h, "ip-tunnel", nmc_fields_setting_ip_tunnel);
@@ -171,17 +171,18 @@ usage (void)
 	g_printerr (_("Usage: nmcli [OPTIONS] OBJECT { COMMAND | help }\n"
 	              "\n"
 	              "OPTIONS\n"
-	              "  -t[erse]                                   terse output\n"
-	              "  -p[retty]                                  pretty output\n"
-	              "  -m[ode] tabular|multiline                  output mode\n"
-	              "  -c[olors] auto|yes|no                      whether to use colors in output\n"
-	              "  -f[ields] <field1,field2,...>|all|common   specify fields to output\n"
-	              "  -e[scape] yes|no                           escape columns separators in values\n"
-	              "  -a[sk]                                     ask for missing parameters\n"
-	              "  -s[how-secrets]                            allow displaying passwords\n"
-	              "  -w[ait] <seconds>                          set timeout waiting for finishing operations\n"
-	              "  -v[ersion]                                 show program version\n"
-	              "  -h[elp]                                    print this help\n"
+	              "  -t[erse]                                       terse output\n"
+	              "  -p[retty]                                      pretty output\n"
+	              "  -m[ode] tabular|multiline                      output mode\n"
+	              "  -c[olors] auto|yes|no                          whether to use colors in output\n"
+	              "  -f[ields] <field1,field2,...>|all|common       specify fields to output\n"
+	              "  -g[et-values] <field1,field2,...>|all|common   shortcut for -m tabular -t -f\n"
+	              "  -e[scape] yes|no                               escape columns separators in values\n"
+	              "  -a[sk]                                         ask for missing parameters\n"
+	              "  -s[how-secrets]                                allow displaying passwords\n"
+	              "  -w[ait] <seconds>                              set timeout waiting for finishing operations\n"
+	              "  -v[ersion]                                     show program version\n"
+	              "  -h[elp]                                        print this help\n"
 	              "\n"
 	              "OBJECT\n"
 	              "  g[eneral]       NetworkManager's general status and operations\n"
@@ -218,9 +219,9 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 	if (argc > 1 && nm_streq (argv[1], "--complete-args")) {
 		nmc->complete = TRUE;
 		argv[1] = argv[0];
-		argc--; argv++;
+		next_arg (nmc, &argc, &argv);
 	}
-	argc--; argv++;
+	next_arg (nmc, &argc, &argv);
 
 	/* parse options */
 	while (argc) {
@@ -230,15 +231,15 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 
 		if (argc == 1 && nmc->complete) {
 			nmc_complete_strings (opt, "--terse", "--pretty", "--mode", "--colors", "--escape",
-			                           "--fields", "--nocheck", "--ask", "--show-secrets",
-			                           "--wait", "--version", "--help", NULL);
+			                           "--fields", "--nocheck", "--get-values",
+			                            "--wait", "--version", "--help", NULL);
 		}
 
 		if (opt[1] == '-') {
 			opt++;
 			/* '--' ends options */
 			if (opt[1] == '\0') {
-				argc--; argv++;
+				next_arg (nmc, &argc, &argv);
 				break;
 			}
 		}
@@ -271,11 +272,13 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 				nmc->print_output = NMC_PRINT_PRETTY;
 		} else if (matches (opt, "-mode")) {
 			nmc->mode_specified = TRUE;
-			if (next_arg (&argc, &argv) != 0) {
+			if (!argc) {
 				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
 				return FALSE;
 			}
+			argc--;
+			argv++;
 			if (argc == 1 && nmc->complete)
 				nmc_complete_strings (argv[0], "tabular", "multiline", NULL);
 			if (matches (argv[0], "tabular"))
@@ -288,11 +291,13 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 				return FALSE;
 			}
 		} else if (matches (opt, "-colors")) {
-			if (next_arg (&argc, &argv) != 0) {
+			if (!argc) {
 				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
 				return FALSE;
 			}
+			argc--;
+			argv++;
 			if (argc == 1 && nmc->complete)
 				nmc_complete_strings (argv[0], "yes", "no", "auto", NULL);
 			if (matches (argv[0], "auto"))
@@ -307,11 +312,13 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 				return FALSE;
 			}
 		} else if (matches (opt, "-escape")) {
-			if (next_arg (&argc, &argv) != 0) {
+			if (!argc) {
 				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
 				return FALSE;
 			}
+			argc--;
+			argv++;
 			if (argc == 1 && nmc->complete)
 				nmc_complete_strings (argv[0], "yes", "no", NULL);
 			if (matches (argv[0], "yes"))
@@ -324,27 +331,44 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 				return FALSE;
 			}
 		} else if (matches (opt, "-fields")) {
-			if (next_arg (&argc, &argv) != 0) {
+			if (!argc) {
 				g_string_printf (nmc->return_text, _("Error: fields for '%s' options are missing."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
 				return FALSE;
 			}
+			argc--;
+			argv++;
 			if (argc == 1 && nmc->complete)
 				complete_fields (argv[0]);
 			nmc->required_fields = g_strdup (argv[0]);
+		} else if (matches (opt, "-get-values")) {
+			if (!argc) {
+				g_string_printf (nmc->return_text, _("Error: fields for '%s' options are missing."), opt);
+				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
+				return FALSE;
+			}
+			argc--;
+			argv++;
+			if (argc == 1 && nmc->complete)
+				complete_fields (argv[0]);
+			nmc->required_fields = g_strdup (argv[0]);
+			nmc->print_output = NMC_PRINT_TERSE;
+			/* We want fixed tabular mode here, but just set the mode specified and rely on the initialization
+			 * in nmc_init: in this way we allow use of "-m multiline" to swap the output mode also if placed
+			 * before the "-g <field>" option (-g may be still more practical and easy to remember than -t -f).
+			*/
+			nmc->mode_specified = TRUE;
 		} else if (matches (opt, "-nocheck")) {
 			/* ignore for backward compatibility */
-		} else if (matches (opt, "-ask")) {
-			nmc->ask = TRUE;
-		} else if (matches (opt, "-show-secrets")) {
-			nmc->show_secrets = TRUE;
 		} else if (matches (opt, "-wait")) {
 			unsigned long timeout;
-			if (next_arg (&argc, &argv) != 0) {
+			if (!argc) {
 				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
 				return FALSE;
 			}
+			argc--;
+			argv++;
 			if (!nmc_string_to_uint (argv[0], TRUE, 0, G_MAXINT, &timeout)) {
 				g_string_printf (nmc->return_text, _("Error: '%s' is not a valid timeout for '%s' option."),
 						 argv[0], opt);
@@ -365,8 +389,7 @@ process_command_line (NmCli *nmc, int argc, char **argv)
 			nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
 			return FALSE;
 		}
-		argc--;
-		argv++;
+		next_arg (nmc, &argc, &argv);
 	}
 
 	/* Now run the requested command */
