@@ -24,11 +24,11 @@
  * access points displayed by "nmtui connect".
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <stdlib.h>
 
-#include <NetworkManager.h>
+#include "NetworkManager.h"
 
 #include "nmtui.h"
 #include "nmt-connect-connection-list.h"
@@ -107,6 +107,7 @@ static const char *device_sort_order[] = {
 	NM_SETTING_BOND_SETTING_NAME,
 	NM_SETTING_TEAM_SETTING_NAME,
 	NM_SETTING_BRIDGE_SETTING_NAME,
+	NM_SETTING_IP_TUNNEL_SETTING_NAME,
 	"NMDeviceModem",
 	"NMDeviceBt"
 };
